/*
 * Copyright (C) 2015 Infineon Technologies AG. All rights reserved.
 *
 * Infineon Technologies AG (Infineon) is supplying this software for use with
 * Infineon's microcontrollers.
 * This file can be freely distributed within development tools that are
 * supporting such microcontrollers.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * ATTENTION:
 * This example demonstrates the ethercat peripheral of XMC 4800. For this purpose an
 * ethercat slave node implemented inside a library is included into this project.
 * This library is for demonstration purpose within this example only and is
 * not certified, not safe and not meant to be used in any other context than here.
 *
 */


/**
 * @file
 * @date 2 Dec, 2015
 * @version 1.0.0
 *
 * @brief XMC4800 Relax Kit EtherCat demo example
 *
 * This example initializes a generic slave node and makes only sense to be evaluated with a ethercat host.
 * The generic slave implements 8x1bit and 4x16bit on input-side and
 * 8x1bit and 4x16bit on output-side.
 *
 * Button1 (P15.13) and Button2 (P15.12) on your XMC4800 Relax EtherCAT Kit can be used to control of 2x1bit
 * master input (IN_GENERIC->IN_BOOL1/2).
 *
 * On master output side 8x1bit (OUT_GENERIC->OUT_BOOL1-8) are used to control the state of 8 LEDs on your
 * XMC4800 Relax EtherCAT Kit. Furthermore a 16bit integer (OUT_GENERIC->OUT_INT1) is used to dimm LED2 on
 * your base board via PWM.
 *
 * ATTENTION:
 * This example demonstrates the ethercat peripheral of XMC 4800. For this purpose an
 * ethercat slave node implementation inside the library libEthCatGenSlv.a is included into this project.
 * This library is for demonstration purpose within this example only
 * and is not certified, not conform and not planned to be used in any other context than this
 * example here.
 * For your final product you have to use a slave stack which is prepared for getting certified.
 * On the market such solutions are available.
 * Further information on development systems and tools you find here:
 * https://www.ethercat.org/en/products.html
 * History <br>
 *
 * Version 1.0.0 Initial <br>
 *
 */

#include <DAVE.h>               //Declarations from DAVE Code Generation (includes SFR declaration)
#include "gen_objectDict.h"

#define P_LED1 P1_14
#define P_LED2 P0_14
#define P_LED3 P3_14
#define P_LED4 P0_15
#define P_LED5 P1_2
#define P_LED6 P3_13
#define P_LED7 P5_3
#define P_LED8 P3_11

#define MAP2LEVEL(x) ((x==0)?XMC_GPIO_OUTPUT_LEVEL_HIGH:XMC_GPIO_OUTPUT_LEVEL_LOW)

uint8_t timerCounter = 0;

void Init_ECAT_Adapt_LED ()
{
  /* Set mode of all LED ports to push-pull and output level to low*/
  XMC_GPIO_SetMode(P_LED1, XMC_GPIO_MODE_OUTPUT_PUSH_PULL);
  XMC_GPIO_SetOutputHigh(P_LED1);
  XMC_GPIO_SetMode(P_LED2, XMC_GPIO_MODE_OUTPUT_PUSH_PULL);
  XMC_GPIO_SetOutputHigh(P_LED2);
  XMC_GPIO_SetMode(P_LED3, XMC_GPIO_MODE_OUTPUT_PUSH_PULL);
  XMC_GPIO_SetOutputHigh(P_LED3);
  XMC_GPIO_SetMode(P_LED4, XMC_GPIO_MODE_OUTPUT_PUSH_PULL);
  XMC_GPIO_SetOutputHigh(P_LED4);
  XMC_GPIO_SetMode(P_LED5, XMC_GPIO_MODE_OUTPUT_PUSH_PULL);
  XMC_GPIO_SetOutputHigh(P_LED5);
  XMC_GPIO_SetMode(P_LED6, XMC_GPIO_MODE_OUTPUT_PUSH_PULL);
  XMC_GPIO_SetOutputHigh(P_LED6);
  XMC_GPIO_SetMode(P_LED7, XMC_GPIO_MODE_OUTPUT_PUSH_PULL);
  XMC_GPIO_SetOutputHigh(P_LED7);
  XMC_GPIO_SetMode(P_LED8, XMC_GPIO_MODE_OUTPUT_PUSH_PULL);
  XMC_GPIO_SetOutputHigh(P_LED8);
}

void Init_Relax_Button ()
{
  /* INITIALIZE BUTTON1 ON PORT 5.13 FOR INPUT */
  /* Set mode to input tristate */
  XMC_GPIO_SetMode(P15_13, XMC_GPIO_MODE_INPUT_TRISTATE);
  /* Enable digital input. Only needed because P15.13 is an analog port */
  XMC_GPIO_EnableDigitalInput(P15_13);

  /* INITIALIZE BUTTON2 ON PORT 5.12 FOR INPUT */
  /* Set mode to input tristate */
  XMC_GPIO_SetMode(P15_12, XMC_GPIO_MODE_INPUT_TRISTATE);
  /* Enable digital input. Only needed because P15.12 is an analog port */
  XMC_GPIO_EnableDigitalInput(P15_12);
}



void timerInit(
        void
    )
{
	uint32_t returnCode;

    /* start hardware timer */
    returnCode = SysTick_Config(
         /* SystemCoreClock / Ticks per second */
            SystemCoreClock / (1000000UL / (4000UL) )
            );
    if (returnCode != 0) { /* Check return code for errors */
        /* Error Handling */
    }
}

void SysTick_Handler(
        void
    )
{
    timerCounter++;
}

void userAppl()
{
	static uint8_t direction = 0;
	static uint8_t pattern = 0;
	static uint8_t mode = 0;

	/* INPUT PROCESSING */
	/*Check Button 1 and set IN_GEN_Bit1 which is sent to master accordingly*/
//	if (XMC_GPIO_GetInput(P15_13)) {
//		buttons = buttons & 0xFE;
//		if(mode == 1) {
//			pattern = 0x00;
//		}
//	} else {
//		mode = 1;
//		pattern = 0xFF;
//		buttons = buttons | 0x01;
//	}
//
//	/*Check Button 2 and set IN_GEN_Bit2 which is sent to master accordingly*/
//	if (XMC_GPIO_GetInput(P15_12)) {
//		buttons = buttons & 0xFD;
//	} else {
//		mode = 0;
//		buttons = buttons | 0x02;
//	}

	buttons.buttonOne = ~XMC_GPIO_GetInput(P15_12);
	buttons.buttonTwo = ~XMC_GPIO_GetInput(P15_13);

	if(mode == 0) {
		switch(led_pattern) {
		case 0:
			// all off
			pattern = 0;
			break;
		case 1:
			// all on
			pattern = 0xFF;
			break;
		case 2:
			// one on, one off, one on..
			if(timerCounter == 255) {
				if(pattern == 170) {
					pattern = 85;
				} else {
					pattern = 170;
				}
				timerCounter = 0;
			}
			break;
		case 3:
			// running light starting at the beginning each time
			pattern = (1 << (timerCounter*2 / 32));
			if(timerCounter == 128) {
				timerCounter = 0;
			}
			break;
		case 4:
			// running light from a to b and back
			if(direction) {
				pattern = (1 << (timerCounter*2 / 32));
			} else {
				pattern = (0x80 >> (timerCounter*2 / 32));
			}
			if(timerCounter == 128) {
				timerCounter = 0;
				direction = ~direction;
			}
			break;
		default:
			pattern = 0;
			break;
		}
	}

	XMC_GPIO_SetOutputLevel(P_LED1, MAP2LEVEL((pattern & (0x01 << 0))));
	XMC_GPIO_SetOutputLevel(P_LED2, MAP2LEVEL((pattern & (0x01 << 1))));
	XMC_GPIO_SetOutputLevel(P_LED3, MAP2LEVEL((pattern & (0x01 << 2))));
	XMC_GPIO_SetOutputLevel(P_LED4, MAP2LEVEL((pattern & (0x01 << 3))));
	XMC_GPIO_SetOutputLevel(P_LED5, MAP2LEVEL((pattern & (0x01 << 4))));
	XMC_GPIO_SetOutputLevel(P_LED6, MAP2LEVEL((pattern & (0x01 << 5))));
	XMC_GPIO_SetOutputLevel(P_LED7, MAP2LEVEL((pattern & (0x01 << 6))));
	XMC_GPIO_SetOutputLevel(P_LED8, MAP2LEVEL((pattern & (0x01 << 7))));
}

int main(void)
{
  DAVE_STATUS_t status;
  status = DAVE_Init();                     /* Initialization of DAVE APPs  */

  Init_ECAT_Adapt_LED ();                   /* Initialize the ports which drive LEDs on EtherCat shield*/
  Init_Relax_Button();                      /* Initialize buttons on XMC4800 */

  timerInit();

  if(status == DAVE_STATUS_FAILURE)
  {
    /* Placeholder for error handler code. The while loop below can be replaced with an user error handler. */
    XMC_DEBUG("DAVE APPs initialization failed\n");
    while(1U)
    {
    }
  }

  /* Placeholder for user application code. The while loop below can be replaced with user application code. */
  while(1U)
  {
    MainLoop();
    userAppl();
  }
}
